/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file	    : main.c
  * @brief	    : Main program body
  ******************************************************************************
  */
/* USER CODE END Header */
// Includes ------------------------------------------------------------------
#include "main.h"
#include "cmsis_os.h"
#include "usb_host.h"

// Private includes ----------------------------------------------------------
/* USER CODE BEGIN Includes */
#include "arm_math.h"
#include "hd44780.h"
#include "morse.h"
#include "keyer.h"
#include "threads.h"
#include "configuration.h"
#include "usbh_hid.h"
#include "usbh_hid_parser.h"
/* USER CODE END Includes */

// Private typedef -----------------------------------------------------------
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

// Private define ------------------------------------------------------------
/* USER CODE BEGIN PD */
// #define DEBUG_HEAP
// #define DEBUG_MOUSE
// #define DEBUG_KBD
/* USER CODE END PD */

// Private macro -------------------------------------------------------------
/* USER CODE BEGIN PM */

/* USER CODE END PM */

// Private variables ---------------------------------------------------------
 I2S_HandleTypeDef hi2s2;
I2S_HandleTypeDef hi2s3;
DMA_HandleTypeDef hdma_spi2_rx;
DMA_HandleTypeDef hdma_spi3_tx;

IWDG_HandleTypeDef hiwdg;

RTC_HandleTypeDef hrtc;

SPI_HandleTypeDef hspi1;

TIM_HandleTypeDef htim10;

osThreadId defaultTaskHandle;
/* USER CODE BEGIN PV */
volatile int  display_timeout;
char          current_string[MAX_STRING_SIZE];
volatile int  string_wptr;
volatile int  fn_key;
/* USER CODE END PV */

// Private function prototypes -----------------------------------------------
void SystemClock_Config(void);
void PeriphCommonClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_SPI1_Init(void);
static void MX_RTC_Init(void);
static void MX_DMA_Init(void);
static void MX_TIM10_Init(void);
static void MX_I2S2_Init(void);
static void MX_I2S3_Init(void);
static void MX_IWDG_Init(void);
void StartDefaultTask(void const * argument);

/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

// Private user code ---------------------------------------------------------
/* USER CODE BEGIN 0 */
//#define DEBUG_HEAP
void
USBH_HID_EventCallback( USBH_HandleTypeDef * phost )
{
#ifdef DEBUG_HEAP
  HeapStats_t   HeapStats;
#endif
  int           X_Val, Y_Val;
  if( USBH_HID_GetDeviceType( phost ) == HID_MOUSE )    // if the HID is Mouse
  {
    HID_MOUSE_Info_TypeDef *Mouse_Info;
    Mouse_Info = USBH_HID_GetMouseInfo( phost );        // Get the info
    X_Val = Mouse_Info->x;      // get the x value
    Y_Val = Mouse_Info->y;      // get the y value
    if( X_Val > 127 )
      X_Val -= 255;
    if( Y_Val > 127 )
      Y_Val -= 255;
#ifdef DEBUG_MOUSE
    printf( "X=%d, Y=%d, Button1=%d, Button2=%d, Button3=%d\n",
        X_Val, Y_Val, Mouse_Info->buttons[0], Mouse_Info->buttons[1],
        Mouse_Info->buttons[2] );
#endif
  }
  if( USBH_HID_GetDeviceType( phost ) == HID_KEYBOARD ) // if the HID is Mouse
  {
    bool          shift, ctrl, alt;
    char         *str;
    u8            key;
    int           len;

    HID_KEYBD_Info_TypeDef *Keyboard_Info;
    Keyboard_Info = USBH_HID_GetKeybdInfo( phost );     // get the info
    key = USBH_HID_GetASCIICode( Keyboard_Info );       // get the key pressed
    if( key >= 'a' && key <= 'z' )
      key &= 0xdf;              // upper case
    if( key == '\\' )           // ignore backslash
      key = 0;
#ifdef DEBUG_KBD
    printf( "Key Pressed = %c [%02x] Code = %02x\n", key, key,
        Keyboard_Info->keys[0] );
#endif
    shift = Keyboard_Info->lshift || Keyboard_Info->rshift;
    ctrl = Keyboard_Info->lctrl || Keyboard_Info->rctrl;
    alt = Keyboard_Info->lalt || Keyboard_Info->ralt;
    if( key )
    {
      if( ctrl )                // Ctrl-Letter commands
      {
        switch ( key )
        {
          case '`':
#ifdef DEBUG_HEAP
            printf( "Unused stack per thread:\n" );
            printf( "Default %ld USB %ld Init LCD %ld Beacon %ld Tx CW %ld Tx Decode %ld LCD %ld Options %ld\n",
                uxTaskGetStackHighWaterMark( defaultTaskHandle ),
                uxTaskGetStackHighWaterMark( phost->thread ),
                uxTaskGetStackHighWaterMark( thread_init_lcd ),
                uxTaskGetStackHighWaterMark( thread_beacon ),
                uxTaskGetStackHighWaterMark( thread_tx_cw ),
                uxTaskGetStackHighWaterMark( thread_tx_decode ),
		uxTaskGetStackHighWaterMark( thread_lcd ),
                uxTaskGetStackHighWaterMark( thread_options ) );
#ifdef ENABLE_CW_DECODE
            printf( "Rx Decode %ld Timing Decode %ld\n",
                uxTaskGetStackHighWaterMark( thread_rx_decode ),
                uxTaskGetStackHighWaterMark( thread_timing_decode ) );
#endif                          // ENABLE_CW_DECODE
#endif                          // DEBUG_HEAP
            break;
        }
      }
      else if( alt )            // Alt-Letter commands
      {
        switch ( key )
        {
          case 'S':
            main_mode = M_SAVE_OPT;
            TaskNotifyISR( thread_options, NOTIFY_SAVE_OPTIONS );
            display_timeout = SAVEOPT_TIMEOUT;
            TaskNotifyISR( thread_lcd, NOTIFY_SAVEOPT_DISPLAY );
            break;
        }
      }
      else
      {
        switch ( key )
        {
          case '\r':           // backspace
            if( main_mode == M_ENTER_STRING )
            {
              if( string_wptr > 0 )
              {
                string_wptr--;
                TaskNotifyISR( thread_lcd, NOTIFY_STRING_DISPLAY );
              }
            }
            break;
          case '\n':           // carriage return
            if( main_mode == M_ENTER_STRING )
            {
              while( string_wptr && current_string[string_wptr - 1] == ' ' )
                string_wptr--;
              current_string[string_wptr] = 0;
              main_mode = M_KBD_TX_RX;
              TaskNotifyISR( thread_options, NOTIFY_SAVE_STRING );
            }
            main_mode = M_KBD_TX_RX;
            break;
          case '\t':           // tab // Pause Sending Immediately
          case '#':
          case '%':
          case '^':
          case '\\':
          case '|':
          case '<':
          case '>':
          case '[':
          case ']':
          case '{':
          case '}':
          case '$':
          case '*':
            break;
          case '~':
#ifdef DEBUG_HEAP
            vPortGetHeapStats( &HeapStats );
            printf( "Total free heap space %d bytes\n",
                HeapStats.xAvailableHeapSpaceInBytes );
            printf( "Minimum heap space %d bytes\n",
                HeapStats.xMinimumEverFreeBytesRemaining );
            printf( "%d allocs %d frees\n",
                HeapStats.xNumberOfSuccessfulAllocations,
                HeapStats.xNumberOfSuccessfulFrees );
#endif
            break;
          default:
            if( main_mode == M_ENTER_STRING )
            {
              if( string_wptr < MAX_STRING_SIZE - 1 )
              {
                current_string[string_wptr++] = key;
                TaskNotifyISR( thread_lcd, NOTIFY_STRING_DISPLAY );
              }
              break;
            }
            main_mode = M_KBD_TX_RX;
            cw_tx_buffer[cw_wptr++] = key;
            cw_wptr %= CW_BUF_LEN;
            TaskNotifyISR( thread_lcd, NOTIFY_LETTER );
            TaskNotifyISR( thread_tx_cw, NOTIFY_CHARS );
            break;
        }
      }
    }
    else                        // special keys
    {
      switch ( Keyboard_Info->keys[0] )
      {
        case 0x29:             // ESC
          main_mode = M_KBD_TX_RX;
          // interrupt any transmission
          memset( cw_tx_buffer, 0, CW_BUF_LEN );
          cw_wptr = cw_rptr = 0;
          TaskNotifyISR( thread_lcd, NOTIFY_CLEAR );
          break;
        case 0x3a:             // F1
        case 0x3b:             // F2
        case 0x3c:             // F3
        case 0x3d:             // F4
        case 0x3e:             // F5
        case 0x3f:             // F6
        case 0x40:             // F7
        case 0x41:             // F8
        case 0x42:             // F9
        case 0x43:             // F10
        case 0x44:             // F11
        case 0x45:             // F12
          fn_key = Keyboard_Info->keys[0] - 0x39;
          if( ctrl )
          {
            main_mode = M_ENTER_STRING;
#ifdef ENABLE_CW_DECODE
            Options.decoder_mode = DECODER_OFF;
#endif
            if( Options.string_memory[fn_key - 1] )
            {
              string_wptr = strlen( Options.string_memory[fn_key - 1] );
              strncpy( current_string, Options.string_memory[fn_key - 1],
                  string_wptr );
            }
            else
            {
              string_wptr = 0;
            }
            TaskNotifyISR( thread_lcd, NOTIFY_STRING_DISPLAY );
          }
          else if( alt )
          {
            ;
          }
          else if( shift )
          {
            ;
          }
          else
          {
            if( main_mode == M_KBD_TX_RX )
            {
              if( Options.string_memory[fn_key - 1] )
              {
                len = strlen( Options.string_memory[fn_key - 1] );
                len = len > ( MAX_STRING_SIZE - 1 ) ?
                    MAX_STRING_SIZE - 1 : len;
                strncpy( current_string, Options.string_memory[fn_key - 1],
                    len );
                current_string[len++] = ' ';
                str = current_string;
                while( len-- )
                {
                  cw_tx_buffer[cw_wptr++] = *str++;
                  cw_wptr %= CW_BUF_LEN;
                }
                TaskNotifyISR( thread_lcd, NOTIFY_LETTER );
                TaskNotifyISR( thread_tx_cw, NOTIFY_CHARS );
              }
            }
          }
          break;
        case 0x4c:             // Delete
        case 0x4f:             // Arrow Right
        case 0x50:             // Arrow Left
        case 0x51:             // Arrow Down
        case 0x52:             // Arrow Up
        case 0x39:             // Caps Lock
          break;
      }
    }
  }
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  int           watchdog_reset = __HAL_RCC_GET_FLAG( RCC_FLAG_IWDGRST );
  UNUSED( watchdog_reset );
  __HAL_RCC_CLEAR_RESET_FLAGS(  );
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

// Configure the peripherals common clocks
  PeriphCommonClock_Config();

  /* USER CODE BEGIN SysInit */
  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_SPI1_Init();
  MX_RTC_Init();
  MX_DMA_Init();
  MX_TIM10_Init();
  MX_I2S2_Init();
  MX_I2S3_Init();
  MX_IWDG_Init();
  /* USER CODE BEGIN 2 */
  HAL_TIM_Base_Start_IT( &htim10 );
  /* USER CODE END 2 */

  /* USER CODE BEGIN RTOS_MUTEX */
  // osMutexDef(Mutex1);
  // h_mux1 = osMutexCreate(osMutex(Mutex1));
  /* USER CODE END RTOS_MUTEX */

  /* USER CODE BEGIN RTOS_SEMAPHORES */
  // osSemaphoreDef(Sem_CW);
  // hsem_CW = osSemaphoreCreate(osSemaphore(Sem_CW), 1);
  // osSemaphoreRelease(hsem_CW);
  // osSemaphoreWait( hsem_CW, 0 );
  /* USER CODE END RTOS_SEMAPHORES */

  /* USER CODE BEGIN RTOS_TIMERS */
  /* start timers, add new ones, ... */
  /* USER CODE END RTOS_TIMERS */

  /* USER CODE BEGIN RTOS_QUEUES */
  /* add queues, ... */
  /* USER CODE END RTOS_QUEUES */

  /* Create the thread(s) */
  /* definition and creation of defaultTask */
  osThreadDef(defaultTask, StartDefaultTask, osPriorityNormal, 0, 96);
  defaultTaskHandle = osThreadCreate(osThread(defaultTask), NULL);

  /* USER CODE BEGIN RTOS_THREADS */
  // init_lcd_thread
  osThreadDef( init_lcd, init_lcd_thread, osPriorityNormal, 0, 64 );
  thread_init_lcd = osThreadCreate( osThread( init_lcd ), NULL );
  // beacon_thread
  osThreadDef( beacon, beacon_thread, osPriorityNormal, 0, 64 );
  thread_beacon = osThreadCreate( osThread( beacon ), NULL );
  // options_thread
  osThreadDef( options, options_thread, osPriorityNormal, 0, 96 );
  thread_options = osThreadCreate( osThread( options ), NULL );
  // lcd_thread
  osThreadDef( display, lcd_thread, osPriorityNormal, 0, 128 );
  thread_lcd = osThreadCreate( osThread( display ), NULL );
  // tx_cw_thread
  osThreadDef( tx_cw, tx_cw_thread, osPriorityNormal, 0, 96 );
  thread_tx_cw = osThreadCreate( osThread( tx_cw ), NULL );
  // tx_decode_thread
  osThreadDef( tx_decode, tx_decode_thread, osPriorityNormal, 0, 64 );
  thread_tx_decode = osThreadCreate( osThread( tx_decode ), NULL );
#ifdef ENABLE_CW_DECODE
  // rx_decode_thread
  osThreadDef( rx_decode, rx_decode_thread, osPriorityNormal, 0, 168 );
  thread_rx_decode = osThreadCreate( osThread( rx_decode ), NULL );
  // timing_decode_thread
  osThreadDef( timing_decode, timing_decode_thread, osPriorityNormal, 0,
      168 );
  thread_timing_decode = osThreadCreate( osThread( timing_decode ), NULL );
#endif
  /* USER CODE END RTOS_THREADS */

  /* Start scheduler */
  osKernelStart();

  // We should never get here as control is now taken by the scheduler
  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while( 1 )
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

    HAL_PWR_EnterSLEEPMode( PWR_MAINREGULATOR_ON, PWR_SLEEPENTRY_WFI );
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief Peripherals Common Clock Configuration
  * @retval None
  */
void PeriphCommonClock_Config(void)
{
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct = {0};

  /** Initializes the peripherals clock
  */
  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_I2S;
  PeriphClkInitStruct.PLLI2S.PLLI2SN = 256;
  PeriphClkInitStruct.PLLI2S.PLLI2SR = 2;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief I2S2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2S2_Init(void)
{

  /* USER CODE BEGIN I2S2_Init 0 */

  /* USER CODE END I2S2_Init 0 */

  /* USER CODE BEGIN I2S2_Init 1 */

  /* USER CODE END I2S2_Init 1 */
  hi2s2.Instance = SPI2;
  hi2s2.Init.Mode = I2S_MODE_MASTER_RX;
  hi2s2.Init.Standard = I2S_STANDARD_PHILIPS;
  hi2s2.Init.DataFormat = I2S_DATAFORMAT_24B;
  hi2s2.Init.MCLKOutput = I2S_MCLKOUTPUT_DISABLE;
  hi2s2.Init.AudioFreq = I2S_AUDIOFREQ_16K;
  hi2s2.Init.CPOL = I2S_CPOL_LOW;
  hi2s2.Init.ClockSource = I2S_CLOCK_PLL;
  hi2s2.Init.FullDuplexMode = I2S_FULLDUPLEXMODE_DISABLE;
  if (HAL_I2S_Init(&hi2s2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2S2_Init 2 */

  /* USER CODE END I2S2_Init 2 */

}

/**
  * @brief I2S3 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2S3_Init(void)
{

  /* USER CODE BEGIN I2S3_Init 0 */

  /* USER CODE END I2S3_Init 0 */

  /* USER CODE BEGIN I2S3_Init 1 */

  /* USER CODE END I2S3_Init 1 */
  hi2s3.Instance = SPI3;
  hi2s3.Init.Mode = I2S_MODE_MASTER_TX;
  hi2s3.Init.Standard = I2S_STANDARD_PHILIPS;
  hi2s3.Init.DataFormat = I2S_DATAFORMAT_16B;
  hi2s3.Init.MCLKOutput = I2S_MCLKOUTPUT_DISABLE;
  hi2s3.Init.AudioFreq = I2S_AUDIOFREQ_16K;
  hi2s3.Init.CPOL = I2S_CPOL_LOW;
  hi2s3.Init.ClockSource = I2S_CLOCK_PLL;
  hi2s3.Init.FullDuplexMode = I2S_FULLDUPLEXMODE_DISABLE;
  if (HAL_I2S_Init(&hi2s3) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2S3_Init 2 */

  /* USER CODE END I2S3_Init 2 */

}

/**
  * @brief IWDG Initialization Function
  * @param None
  * @retval None
  */
static void MX_IWDG_Init(void)
{

  /* USER CODE BEGIN IWDG_Init 0 */

  /* USER CODE END IWDG_Init 0 */

  /* USER CODE BEGIN IWDG_Init 1 */

  /* USER CODE END IWDG_Init 1 */
  hiwdg.Instance = IWDG;
  hiwdg.Init.Prescaler = IWDG_PRESCALER_16;
  hiwdg.Init.Reload = 4095;
  if (HAL_IWDG_Init(&hiwdg) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN IWDG_Init 2 */

  /* USER CODE END IWDG_Init 2 */

}

/**
  * @brief RTC Initialization Function
  * @param None
  * @retval None
  */
static void MX_RTC_Init(void)
{

  /* USER CODE BEGIN RTC_Init 0 */

  /* USER CODE END RTC_Init 0 */

  RTC_TimeTypeDef sTime = {0};
  RTC_DateTypeDef sDate = {0};

  /* USER CODE BEGIN RTC_Init 1 */

  /* USER CODE END RTC_Init 1 */

  /** Initialize RTC Only
  */
  hrtc.Instance = RTC;
  hrtc.Init.HourFormat = RTC_HOURFORMAT_24;
  hrtc.Init.AsynchPrediv = 127;
  hrtc.Init.SynchPrediv = 255;
  hrtc.Init.OutPut = RTC_OUTPUT_DISABLE;
  hrtc.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
  hrtc.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    Error_Handler();
  }

  /* USER CODE BEGIN Check_RTC_BKUP */

  /* USER CODE END Check_RTC_BKUP */

  /** Initialize RTC and set the Time and Date
  */
  sTime.Hours = 0x0;
  sTime.Minutes = 0x0;
  sTime.Seconds = 0x0;
  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BCD) != HAL_OK)
  {
    Error_Handler();
  }
  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
  sDate.Month = RTC_MONTH_JANUARY;
  sDate.Date = 0x1;
  sDate.Year = 0x0;

  if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BCD) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN RTC_Init 2 */

  /* USER CODE END RTC_Init 2 */

}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_HARD_OUTPUT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * @brief TIM10 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM10_Init(void)
{

  /* USER CODE BEGIN TIM10_Init 0 */

  /* USER CODE END TIM10_Init 0 */

  /* USER CODE BEGIN TIM10_Init 1 */

  /* USER CODE END TIM10_Init 1 */
  htim10.Instance = TIM10;
  htim10.Init.Prescaler = 168-1;
  htim10.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim10.Init.Period = 1000;
  htim10.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim10.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim10) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM10_Init 2 */

  /* USER CODE END TIM10_Init 2 */

}

/**
  * Enable DMA controller clock
  */
static void MX_DMA_Init(void)
{

  // DMA controller clock enable
  __HAL_RCC_DMA1_CLK_ENABLE();

  /* DMA interrupt init */
  /* DMA1_Stream3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Stream3_IRQn, 5, 0);
  HAL_NVIC_EnableIRQ(DMA1_Stream3_IRQn);
  /* DMA1_Stream5_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Stream5_IRQn, 5, 0);
  HAL_NVIC_EnableIRQ(DMA1_Stream5_IRQn);

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(Tx_PTT_GPIO_Port, Tx_PTT_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, Tx_CW_Pin|LCD_RS_Pin|LCD_E_Pin|LCD_D4_Pin
                          |LCD_D5_Pin|LCD_D6_Pin|LCD_D7_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, Tx_Pin|Rx_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : Tx_PTT_Pin */
  GPIO_InitStruct.Pin = Tx_PTT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(Tx_PTT_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : ENC_Sw_Pin Enc_A_Pin Enc_B_Pin Spare_Pin
                           Sw_Left_Pin */
  GPIO_InitStruct.Pin = ENC_Sw_Pin|Enc_A_Pin|Enc_B_Pin|Spare_Pin
                          |Sw_Left_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PAD_L_Pin PAD_R_Pin Sw_Right_Pin */
  GPIO_InitStruct.Pin = PAD_L_Pin|PAD_R_Pin|Sw_Right_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : Tx_CW_Pin LCD_RS_Pin LCD_E_Pin LCD_D4_Pin
                           LCD_D5_Pin LCD_D6_Pin LCD_D7_Pin */
  GPIO_InitStruct.Pin = Tx_CW_Pin|LCD_RS_Pin|LCD_E_Pin|LCD_D4_Pin
                          |LCD_D5_Pin|LCD_D6_Pin|LCD_D7_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : Tx_Pin Rx_Pin */
  GPIO_InitStruct.Pin = Tx_Pin|Rx_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/* USER CODE BEGIN Header_StartDefaultTask */
/**
  * @brief  Function implementing the defaultTask thread.
  * @param  argument: Not used
  * @retval None
  */
/* USER CODE END Header_StartDefaultTask */
void StartDefaultTask(void const * argument)
{
  /* init code for USB_HOST */
  MX_USB_HOST_Init();
  /* USER CODE BEGIN 5 */
  UNUSED( argument );
  /* Infinite loop */
  for( ;; )
  {
    osDelay( 1 );
  }
  /* USER CODE END 5 */
}

/**
  * @brief  Period elapsed callback in non blocking mode
  * @note   This function is called  when TIM9 interrupt took place, inside
  * HAL_TIM_IRQHandler(). It makes a direct call to HAL_IncTick() to increment
  * a global variable "uwTick" used as application time base.
  * @param  htim : TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
  /* USER CODE BEGIN Callback 0 */
  static u32    Enc_A, Enc_B, Enc_Sw, Sw_Left, Sw_Right, Pad_L, Pad_R;
  bool          feEnc_B, feEnc_Sw, feSw_Left, feSw_Right, lvfePad_L, lvfePad_R;
  /* USER CODE END Callback 0 */
  if (htim->Instance == TIM9) {
    HAL_IncTick();
  }
  /* USER CODE BEGIN Callback 1 */
  if( htim->Instance == TIM10 ) // 500Hz interrupt
  {
    HAL_IWDG_Refresh( &hiwdg ); // refresh watchdog
    Enc_A = Enc_A << 1 | pinRead( Enc_A_GPIO_Port, Enc_A_Pin );
    Enc_A &= 0xf;
    Enc_B = Enc_B << 1 | pinRead( Enc_B_GPIO_Port, Enc_B_Pin );
    Enc_B &= 0xf;
    Enc_Sw = Enc_Sw << 1 | pinRead( ENC_Sw_GPIO_Port, ENC_Sw_Pin );
    Enc_Sw &= 0xff;
    Sw_Left = Sw_Left << 1 | pinRead( Sw_Left_GPIO_Port, Sw_Left_Pin );
    Sw_Left &= 0xff;
    Sw_Right = Sw_Right << 1 | pinRead( Sw_Right_GPIO_Port, Sw_Right_Pin );
    Sw_Right &= 0xff;
    Pad_L = Pad_L << 1 | pinRead( PAD_L_GPIO_Port, PAD_L_Pin );
    Pad_R = Pad_R << 1 | pinRead( PAD_R_GPIO_Port, PAD_R_Pin );
    feEnc_B = ( Enc_B == 0xc ); // sense falling edge
    feEnc_Sw = ( Enc_Sw == 0xf0 );      // sense falling edge
    feSw_Left = ( Sw_Left == 0xf0 );    // sense falling edge
    feSw_Right = ( Sw_Right == 0xf0 );  // sense falling edge
    if( Pad_L == 0 )            // sense low level
    {
      Pad_L = 1;                // skip next 32 samples
      lvfePad_L = 1;
    }
    else if( ( Pad_L & 0xff ) == 0xf0 ) // sense falling edge
      lvfePad_L = 1;
    else
      lvfePad_L = 0;

    if( Pad_R == 0 )            // sense low level
    {
      Pad_R = 1;                // skip next 32 samples
      lvfePad_R = 1;
    }
    else if( ( Pad_R & 0xff ) == 0xf0 ) // sense falling edge
      lvfePad_R = 1;
    else
      lvfePad_R = 0;
    if( feEnc_B )
    {
      if( Enc_A == 0 || Enc_A == 0xf )
      {
        if( Enc_A )
          change_val = -1;
        else
          change_val = 1;
        if( Sw_Right == 0 )
          change_val *= 10;
        TaskNotifyISR( thread_options, NOTIFY_CHANGE_OPTIONS );
      }
    }
    // handle switches
    if( ( feSw_Left && Sw_Right == 0 ) || ( feSw_Right && Sw_Left == 0 ) )
    {
      main_mode = M_SAVE_OPT;
      TaskNotifyISR( thread_options, NOTIFY_SAVE_OPTIONS );
      display_timeout = SAVEOPT_TIMEOUT;
      TaskNotifyISR( thread_lcd, NOTIFY_SAVEOPT_DISPLAY );
      return;
    }
    if( feEnc_Sw || feSw_Left )
    {
      if( current_config_mode == CONFIG_PARAMETER )
        current_config_mode = CONFIG_VALUE;
      else
        current_config_mode = CONFIG_PARAMETER;
    }
    if( feEnc_B || feEnc_Sw || feSw_Left || feSw_Right )
    {
      main_mode = M_ENC_CFG;
      display_timeout = PARAMETER_TIMEOUT;
      TaskNotifyISR( thread_lcd, NOTIFY_PARAMETER_DISPLAY );
      return;
    }
    if( lvfePad_L || lvfePad_R )
    {
      main_mode = M_PADDLE_TX_RX;
      TaskNotifyISR( thread_tx_cw, NOTIFY_PADDLE );
      return;
    }
    if( main_mode == M_ENC_CFG || main_mode == M_SAVE_OPT )
    {
      if( display_timeout )
      {
        display_timeout--;
        if( !display_timeout )
        {
          main_mode = M_KBD_TX_RX;
          TaskNotifyISR( thread_lcd, NOTIFY_CLEAR | NOTIFY_CHARS );
          return;
        }
      }
    }
  }
  /* USER CODE END Callback 1 */
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq(  );
  while( 1 )
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
