#include "main.h"
#include "hal_i2s.h"
#include "keyer.h"
#include "morse.h"
#include "threads.h"
#include "cmsis_os.h"
#include "configuration.h"

short         sine_wave[DECAY_STEPS][2*MAX_SINE_SAMPLES];
volatile int  sine_wave_decay;
volatile enum paddle_states paddle_state;
volatile enum main_modes main_mode;
u32	      mic_samples[2][2*SAMPLE_BUF_LEN];	
u16           cw_events[MAX_CW_EVENTS];
u32           cw_tm_wprt, cw_tm_rprt;
struct dit_dah_timing morse_timing;

#ifdef ENABLE_CW_DECODE
void
HAL_I2S_RxCpltCallback( I2S_HandleTypeDef * hi2s )
{
  if( hi2s == &hi2s2 )
  {
    //pinSet( Rx_GPIO_Port, Rx_Pin );
    if( Options.decoder_mode == DECODER_OFF )
      HAL_I2S_DMAStop( hi2s );
    //pinReset( Rx_GPIO_Port, Rx_Pin );
    TaskNotifyISR( thread_rx_decode, NOTIFY_DFT_BUF0 );
  }
}

void
HAL_I2S_RxM1CpltCallback( I2S_HandleTypeDef * hi2s )
{
  if( hi2s == &hi2s2 )
  {
    //pinSet( Tx_GPIO_Port, Tx_Pin );
    if( Options.decoder_mode == DECODER_OFF )
      HAL_I2S_DMAStop( hi2s ); 
    //pinReset( Tx_GPIO_Port, Tx_Pin );
    TaskNotifyISR( thread_rx_decode, NOTIFY_DFT_BUF0 );
    
  }
}
#endif // ENABLE_CW_DECODE 

void
HAL_I2S_TxCpltCallback( I2S_HandleTypeDef * hi2s )
{
  if( hi2s == &hi2s3 )
  {
    if( sine_wave_decay == 0 )
      return;
    sine_wave_decay--;
    if( sine_wave_decay )
      HAL_DMAEx_ChangeMemory( hi2s->hdmatx,
          ( u32 ) sine_wave[DECAY_STEPS - sine_wave_decay], MEMORY0 );
    else
      HAL_I2S_DMAStop( hi2s );
  }
}

void
HAL_I2S_TxM1CpltCallback( I2S_HandleTypeDef * hi2s )
{
  if( hi2s == &hi2s3 )
  {
    if( sine_wave_decay == 0 )
      return;
    sine_wave_decay--;
    if( sine_wave_decay )
      HAL_DMAEx_ChangeMemory( hi2s->hdmatx,
          ( u32 ) sine_wave[DECAY_STEPS - sine_wave_decay], MEMORY1 );
    else
      HAL_I2S_DMAStop( hi2s );
  }
}

void
cw_on( void )
{
  pinSet( Tx_CW_GPIO_Port, Tx_CW_Pin );
  cw_events[cw_tm_wprt++] = 1 << 15 | HAL_GetTick(  );
  cw_tm_wprt %= MAX_CW_EVENTS;
  if( Options.sidetone_volume > MIN_VOLUME_DB )
  {
    if( ( Options.sidetone_mode == SIDETONE_ON ) ||
        ( ( Options.sidetone_mode == SIDETONE_PADDLE_ONLY ) &&
        main_mode == M_PADDLE_TX_RX ) )
    {
      sine_wave_decay = 0;
      HAL_I2S_Transmit_DblBuf_DMA( &hi2s3, (u16 *)sine_wave[0], (u16 *)sine_wave[0],
          2*samples_per_cycle );
    }
  }
}

void
cw_off( void )
{
  pinReset( Tx_CW_GPIO_Port, Tx_CW_Pin );
  cw_events[cw_tm_wprt++] = ~( 1 << 15 ) & HAL_GetTick(  );
  cw_tm_wprt %= MAX_CW_EVENTS;
  sine_wave_decay = DECAY_STEPS;
}

void
send_dit( void )
{
  cw_on(  );
  osDelay( morse_timing.dit );
  cw_off(  );
  osDelay( morse_timing.dit );
}

void
send_dah( void )
{
  cw_on(  );
  osDelay( morse_timing.dah );
  cw_off(  );
  osDelay( morse_timing.dit );
}

static void
send_dits_and_dahs( const u32 symbol )
{
  int bits;

  TaskNotify( thread_lcd, NOTIFY_CURSOR_ON );
  for( bits = 28; bits >= 0; bits -= 4 )
  {
    switch ( ( symbol >> bits ) & 0xf )
    {
      case 1:
        send_dit(  );
        break;
      case 3:
        send_dah(  );
        break;
      default:
        break;
    }
  }
  TaskNotify( thread_lcd, NOTIFY_CURSOR_OFF );
  osDelay( 2 * morse_timing.dit + morse_timing.intercharacter );
}

void
send_cw( u8 cw_char )
{
  u32 	symbol = 0;
#if 0
  printf( "send_char: %02x\n", cw_char );
#endif

  switch ( cw_char )
  {
    case 'A':
      symbol = 0x13;
      break;
    case 'B':
      symbol = 0x3111;
      break;
    case 'C':
      symbol = 0x3131;
      break;
    case 'D':
      symbol = 0x311;
      break;
    case 'E':
      symbol = 0x1;
      break;
    case 'F':
      symbol = 0x1131;
      break;
    case 'G':
      symbol = 0x331;
      break;
    case 'H':
      symbol = 0x1111;
      break;
    case 'I':
      symbol = 0x11;
      break;
    case 'J':
      symbol = 0x1333;
      break;
    case 'K':
      symbol = 0x313;
      break;
    case 'L':
      symbol = 0x1311;
      break;
    case 'M':
      symbol = 0x33;
      break;
    case 'N':
      symbol = 0x31;
      break;
    case 'O':
      symbol = 0x333;
      break;
    case 'P':
      symbol = 0x1331;
      break;
    case 'Q':
      symbol = 0x3313;
      break;
    case 'R':
      symbol = 0x131;
      break;
    case 'S':
      symbol = 0x111;
      break;
    case 'T':
      symbol = 0x3;
      break;
    case 'U':
      symbol = 0x113;
      break;
    case 'V':
      symbol = 0x1113;
      break;
    case 'W':
      symbol = 0x133;
      break;
    case 'X':
      symbol = 0x3113;
      break;
    case 'Y':
      symbol = 0x3133;
      break;
    case 'Z':
      symbol = 0x3311;
      break;

    case '0':
      symbol = 0x33333;
      break;
    case '1':
      symbol = 0x13333;
      break;
    case '2':
      symbol = 0x11333;
      break;
    case '3':
      symbol = 0x11133;
      break;
    case '4':
      symbol = 0x11113;
      break;
    case '5':
      symbol = 0x11111;
      break;
    case '6':
      symbol = 0x31111;
      break;
    case '7':
      symbol = 0x33111;
      break;
    case '8':
      symbol = 0x33311;
      break;
    case '9':
      symbol = 0x33331;
      break;

    case '=':
      symbol = 0x31113;
      break;
    case '/':
      symbol = 0x31131;
      break;
    case '.':
      symbol = 0x131313;
      break;
    case ',':
      symbol = 0x331133;
      break;
    case '!':
      symbol = 0x313133;
      break;
    case '\'':
      symbol = 0x133331;
      break;                    // apostrophe
    case '(':
      symbol = 0x31331;
      break;
    case ')':
      symbol = 0x313313;
      break;
    case '&':
      symbol = 0x13111;
      break;
    case '+':
      symbol = 0x13131;
      break;
    case '-':
      symbol = 0x311113;
      break;
    case '_':
      symbol = 0x113313;
      break;
    case '"':
      symbol = 0x131131;
      break;
    case '@':
      symbol = 0x133131;
      break;

    case ':':
      symbol = 0x333111;
      break;
    case ';':
      symbol = 0x313131;
      break;

    case ' ':
      osDelay( 7 * morse_timing.dit + morse_timing.interword );
      break;

    case '?':
    default:
      symbol = 0x113311;
      break;
  }
  if( symbol )
    send_dits_and_dahs( symbol );
}

char
decode_symbol( u32 symbol )
{
  const char *tree = "ETIANMSURWDKGOHVF_L_PJBXCYZQ__54_3___2&_+____16=/___(_7___8_90"
                     "____________?_____\"__.____@___'__-________;!_)_____,____:_____";
  const char *select = tree;
  int offset = 0, level = 0;
  if( symbol == 0 )
    return '_';
  while( (symbol & 0xf) == 0 ) 
    symbol >>= 4;
  for( ;; ) 
  {
    if( (symbol & 0xf) == 3 ) // dah
     offset++;
    symbol >>= 4;
    if ( (symbol & 0xf) == 0 )
      return select[offset];
    offset <<= 1;
    select += 1 << (level + 1);
    level++;
    if( level > 5 )
      return '_';
  }
  return '_';
}

void
decode_tx_morse_code( void )
{
  static u16    prev_tick, tick;
  u16           cw_on, cw_delta;
  u32           symbol, last_wprt;
  int           insert_space = 0;
  char          ch;
  if( cw_tm_wprt == cw_tm_rprt )
    return;
  if( cw_tm_wprt )
    last_wprt = cw_tm_wprt - 1;
  else
    last_wprt = MAX_CW_EVENTS - 1;
  if( ( 1 << 15 ) & cw_events[last_wprt] )      // still on
    return;
  tick = ~( 1 << 15 ) & HAL_GetTick(  );
  if( ( tick - cw_events[last_wprt] ) < 5 * morse_timing.dit / 2 )      // letter pause must be longer than 2.5 dit
    return;
  if( ( tick - prev_tick ) > 13 * morse_timing.dit / 2 + morse_timing.interword ) // word pause
    insert_space = 1;
  prev_tick = tick;
  symbol = 0;
  while( cw_tm_rprt != cw_tm_wprt )
  {
    tick = cw_events[cw_tm_rprt++];
    cw_tm_rprt %= MAX_CW_EVENTS;
    if( ( 1 << 15 ) & ~tick )
      continue;
    cw_on = ~( 1 << 15 ) & tick;
    tick = cw_events[cw_tm_rprt++];
    cw_tm_rprt %= MAX_CW_EVENTS;
    if( ( 1 << 15 ) & tick )
      continue;
    symbol >>= 4;
    cw_delta = tick - cw_on;
    if( cw_delta > 7 * morse_timing.dit / 4 ) // longer than 1.75 dit is a dah
      symbol |= 3 << 28; 
    else
      symbol |= 1 << 28; 
  }
  ch = decode_symbol( symbol );
  if( insert_space )
  {
    cw_tx_buffer[cw_wptr++] = ' ';
    cw_wptr %= CW_BUF_LEN;
  }
  cw_tx_buffer[cw_wptr++] = ch;
  cw_wptr %= CW_BUF_LEN;
  cw_rptr = cw_wptr;      // sync the cw read pointer    
  TaskNotify( thread_lcd, NOTIFY_CHARS );
}
