/* ----------------------------------------------------------------------
 * Project:	 CMSIS DSP Library
 * Title:	 arm_common_tables.h
 * Description:	 Extern declaration for common tables
 *
 * @version  V1.9.0
 * @date     23 April 2021
 *
 * Target Processor: Cortex-M and Cortex-A cores
 * -------------------------------------------------------------------- */

#ifndef _ARM_COMMON_TABLES_H
#define _ARM_COMMON_TABLES_H

#define ARM_DSP_CONFIG_TABLES
#define ARM_FFT_ALLOW_TABLES
#define ARM_FAST_ALLOW_TABLES
#define ARM_TABLE_TWIDDLECOEF_F32_64
#define ARM_TABLE_BITREVIDX_FLT_64
#define ARM_TABLE_TWIDDLECOEF_RFFT_F32_128
#define ARM_TABLE_SIN_F32

#include "arm_math_types.h"
#include "fast_math_functions.h"

  /* Double Precision Float CFFT twiddles */

  #if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || defined(ARM_TABLE_TWIDDLECOEF_F32_64)
    extern const float32_t twiddleCoef_64[128];
  #endif 

  /* Double Precision Float RFFT twiddles */

  #if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_128)
    extern const float32_t twiddleCoef_rfft_128[128];
  #endif

  /* floating-point bit reversal tables */

  #if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || defined(ARM_TABLE_BITREVIDX_FLT_64)
    #define ARMBITREVINDEXTABLE_64_TABLE_LENGTH ((uint16_t)56)
    extern const uint16_t armBitRevIndexTable64[ARMBITREVINDEXTABLE_64_TABLE_LENGTH];
  #endif /* !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) */

  /* Tables for Fast Math Sine and Cosine */
  #if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FAST_TABLES) || defined(ARM_TABLE_SIN_F32)
    extern const float32_t sinTable_f32[FAST_MATH_TABLE_SIZE + 1];
  #endif /* !defined(ARM_DSP_CONFIG_TABLES) defined(ARM_ALL_FAST_TABLES) */

#endif /*  ARM_COMMON_TABLES_H */
